#!/usr/bin/python3
# -*- coding: utf-8 -*-
# Python Version    : 3.10.X
# Author            : Nishacid

from flask import Flask, request, render_template, send_file, redirect, url_for, flash
from werkzeug.utils import secure_filename
from apscheduler.schedulers.background import BackgroundScheduler
import os
import time
import uuid

# Const variables
UPLOAD_FOLDER = 'uploads'
ALLOWED_EXTENSIONS = {'png', 'jpg', 'jpeg'}
MAX_CONTENT_LENGTH = 5 * 1024 * 1024  # 5MB yeah
CHARTROOUSE = './static/images/chartreuse.png' 
BLACKLIST = [";", "'", "\"", "||", "flag", "&"]

app = Flask(__name__)
app.secret_key = os.urandom(32).hex()
app.config['UPLOAD_FOLDER'] = UPLOAD_FOLDER
app.config['MAX_CONTENT_LENGTH'] = MAX_CONTENT_LENGTH

# Check if the extension is allowed
def allowed_file(filename):
    return '.' in filename and \
           filename.rsplit('.', 1)[1].lower() in ALLOWED_EXTENSIONS

# Blacklisted chars
def is_blacklisted(value):
    for item in BLACKLIST:
        if item in value:
            return True
    return False

# Some security ofc :)
@app.after_request
def add_security_headers(response):
    response.headers['Content-Security-Policy'] = (
        "default-src 'self'; "
        "img-src 'self'; "
        "style-src 'self' 'unsafe-inline'; "
        "script-src 'self' 'unsafe-inline'"
    )
    response.headers['X-Content-Type-Options'] = 'nosniff'
    response.headers['X-Frame-Options'] = 'DENY'
    response.headers['X-XSS-Protection'] = '1; mode=block'
    response.headers['Char'] = 'Treuse'
    response.headers['Strict-Transport-Security'] = 'max-age=31536000; includeSubDomains'
    return response

# How fat is this ?
@app.errorhandler(413)
def request_entity_too_large(error):
    flash('File is too large. Maximum file size is 5MB.')
    return redirect(url_for('index'))

# Home route
@app.route('/')
def index():
    return render_template('index.html')

# Resize route
@app.route('/resize', methods=['POST'])
def resize():
    # Please submit something
    if 'image' not in request.files:
        flash('No file part')
        return redirect(url_for('index'))
    image = request.files['image']
    
    # Don't be shy
    if image.filename == '':
        flash('No selected file')
        return redirect(url_for('index'))

    # Check for allowed extension
    if image and allowed_file(image.filename):
        # C'mon, it's a secure upload here
        filename = secure_filename(image.filename)
        ext = filename.rsplit('.', 1)[1].lower()
        random_filename = f"{uuid.uuid4().hex}.{ext}"
        image_path = os.path.join(app.config['UPLOAD_FOLDER'], random_filename)
        image.save(image_path)

        # Get params
        width = request.form.get('width')
        height = request.form.get('height')

        # I don't want to be hacked hehe
        if is_blacklisted(width) or is_blacklisted(height):
            flash('Invalid width or height.')
            return redirect(url_for('index'))

        # Resize the image
        resized_image_path = os.path.join(app.config['UPLOAD_FOLDER'], 'resized_' + random_filename)
        command = f"convert {image_path} -resize {width}x{height}! {resized_image_path}"
        os.system(command)

        # CHARTROOOUUUSEE
        final_image_path = os.path.join(app.config['UPLOAD_FOLDER'], 'final_' + random_filename)
        watermark_command = f"convert {resized_image_path} {CHARTROOUSE} -gravity center -composite {final_image_path}"
        os.system(watermark_command)

        # oh no, it failed
        if not os.path.exists(final_image_path):
            flash('Image resizing failed')
            return redirect(url_for('index'))

        return redirect(url_for('result', filename='final_' + random_filename))
    else:
        flash('Allowed file types are png, jpg, jpeg')
        return redirect(url_for('index'))

# please grehack-san show me how beautiful it is
@app.route('/result')
def result():
    filename = request.args.get('filename')
    return render_template('result.html', filename=filename)

# where everything is stored
@app.route('/uploads/<filename>')
def uploaded_file(filename):
    file_path = os.path.join(app.config['UPLOAD_FOLDER'], filename)
    # for what are you looking ? 
    if not os.path.exists(file_path):
        flash('File not found')
        return redirect(url_for('index'))
    return send_file(file_path, mimetype='image/png')

# Clean this shit poproszę
def clean_upload_folder():
    now = time.time()
    cutoff = now - (60) 
    files = os.listdir(app.config['UPLOAD_FOLDER'])
    for file in files:
        file_path = os.path.join(app.config['UPLOAD_FOLDER'], file)
        if os.path.isfile(file_path):
            t = os.stat(file_path).st_mtime
            if t < cutoff:
                os.remove(file_path)

if __name__ == '__main__':
    scheduler = BackgroundScheduler()
    scheduler.add_job(func=clean_upload_folder, trigger="interval", minutes=1)
    scheduler.start()
    try:
        app.run(host='0.0.0.0', port=5000, debug=False)
    except (KeyboardInterrupt, SystemExit):
        scheduler.shutdown()
